/*
 * File name: fsm.c
 * Date:      2009/05/03 10:57
 * Author:    Vasek Lorenc
 */

/* -- Includes 
 * */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "fsm.h"

/* -- Typedefs 
 * */

typedef struct {
    char *input_string;
    char *output_string;
    unsigned int input_index, output_index, output_size;
} t_parsing_data;

typedef enum {
    S_NORMAL, S_QUOTE, S_SQUOTE, S_SPACE, S_SLASH1, S_SLASH2, S_SLASH3
} t_states;

/* Global Variables */
t_parsing_data data;

/* -- Functions 
 * */

void init_data(t_parsing_data *data, char *istring, int output_size) {
    data->input_index = 0;
    data->output_index = 0;
    data->input_string = istring;

    data->output_string = (char*) malloc(output_size);
    data->output_size = output_size;
}

/* In order to make the function more complicated.. or universal ;) */
char get_next_char(void) {
    char result;

    if (data.input_index < strlen(data.input_string)) {
        result = data.input_string[data.input_index];
        data.input_index++;
    } else {
        result = EOF;
    }
    return result;
}

/* Simple output function */
void put_next_char(char symbol) {
    putchar(symbol);
}

/*   Description:
 *      main()
 *   Input:
 *      describe the input here
 *   Output:
 *      0
 * */
int main(void) {
    t_states state = S_NORMAL;
    char symbol;

    init_data(&data, "\";\" \\\"&  Testovaci  'retezec   &\\' vskutku' & ; ", 1024);
    printf("Test string: %s\n", data.input_string);

    symbol = get_next_char();

    while (symbol != EOF) {
        printf("[state:] %02d [symbol:] %c [output:] ", state, symbol);

        switch (state) {

            case S_NORMAL:
                switch (symbol) {
                    case ';':
                        put_next_char('\\');
                        break;
                    case '&':
                        put_next_char('\\');
                        break;
                    case '"':
                        state = S_QUOTE;
                        break;
                    case '\'':
                        state = S_SQUOTE;
                        break;
                    case '\\':
                        state = S_SLASH3;
                        break;
                };
                break;

            case S_SQUOTE:
                switch (symbol) {
                    case '\'': 
                        state = S_NORMAL;
                        break;
                    case '\\':
                        state = S_SLASH1;
                        break;
                }
                break;

            case S_QUOTE:
                switch (symbol) {
                    case '"': 
                        state = S_NORMAL;
                        break;
                    case '\\':
                        state = S_SLASH2;
                        break;
                }
                break;

            case S_SLASH1:
                state = S_SQUOTE;
                break;
            case S_SLASH2:
                state = S_QUOTE;
                break;
            case S_SLASH3:
                state = S_NORMAL;
                break;
            case S_SPACE:
                break;
        };

        /* There is no 'output symbol suppression' needed in our example */
        put_next_char(symbol);
        printf("\n");
        /* .. and prepare the next char waiting */
        symbol = get_next_char();
    };

    return 0;
}


/* end of fsm.c */
/* vim: set ts=8 sw=4 tw=0 expandtab cindent smartindent: */

